@@ -146,10 +146,9 @@ nm_phasechange(void *data, int arg)
 static void
 nm_ip_up(void *data, int arg)
 {
-    guint32         pppd_made_up_address = htonl(0x0a404040 + ifunit);
+    guint32         pppd_made_up_address = htonl (0x0a404040 + ifunit);
     ipcp_options    opts                 = ipcp_gotoptions[0];
     ipcp_options    peer_opts            = ipcp_hisoptions[0];
-    ipcp_options    want_opts            = ipcp_wantoptions[0];
     GVariantBuilder builder;
 
     g_return_if_fail(G_IS_DBUS_PROXY(gl.proxy));
@@ -174,23 +173,20 @@ nm_ip_up(void *data, int arg)
                           NM_VPN_PLUGIN_IP4_CONFIG_ADDRESS,
                           g_variant_new_uint32(opts.ouraddr));
 
-    /* Prefer the peer options remote address first, _unless_ it is the
-     * gateway IP address nm-l2tp-services is using
-     * (i.e. want_opts.hisaddr) or pppd made the address up, at which
-     * point prefer the local options remote address, and if that's not
-     * right, use the made-up address as a last resort.
+    /* Prefer the peer options remote address first, _unless_ pppd made the
+     * address up, at which point prefer the local options remote address,
+     * and if that's not right, use the made-up address as a last resort.
      */
-    if (peer_opts.hisaddr && (peer_opts.hisaddr != want_opts.hisaddr)
-        && (peer_opts.hisaddr != pppd_made_up_address)) {
+    if (peer_opts.hisaddr && (peer_opts.hisaddr != pppd_made_up_address)) {
         g_variant_builder_add(&builder,
                               "{sv}",
                               NM_VPN_PLUGIN_IP4_CONFIG_PTP,
-                              g_variant_new_uint32(peer_opts.hisaddr));
-    } else if (opts.hisaddr && (opts.hisaddr != want_opts.hisaddr)) {
+                              g_variant_new_uint32 (peer_opts.hisaddr));
+    } else if (opts.hisaddr){
         g_variant_builder_add(&builder,
                               "{sv}",
                               NM_VPN_PLUGIN_IP4_CONFIG_PTP,
-                              g_variant_new_uint32(opts.hisaddr));
+                              g_variant_new_uint32 (opts.hisaddr));
     } else if (peer_opts.hisaddr == pppd_made_up_address) {
         /* As a last resort, use the made-up address */
         g_variant_builder_add(&builder,
