@@ -89,8 +89,8 @@ typedef struct {
     (G_TYPE_INSTANCE_GET_PRIVATE((o), NM_TYPE_L2TP_PLUGIN, NML2tpPluginPrivate))
 
 #define NM_L2TP_PPPD_PLUGIN       PLUGINDIR "/nm-l2tp-pppd-plugin.so"
-#define NM_L2TP_WAIT_IPSEC        10000 /* 10 seconds */
-#define NM_L2TP_WAIT_PPPD         14000 /* 14 seconds */
+#define NM_L2TP_WAIT_IPSEC        16000 /* 16 seconds */
+#define NM_L2TP_WAIT_PPPD         16000 /* 16 seconds */
 #define L2TP_SERVICE_SECRET_TRIES "l2tp-service-secret-tries"
 
 /* Default Phase 1 (Main Mode) and Phase 2 (Quick Mode) proposals which are a
@@ -592,6 +592,7 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
     g_autofree char *      rundir;
     char                   errorbuf[128];
     gint                   fd = -1;
+    gint64                 max_retries;
     FILE *                 fp;
     struct in_addr         naddr;
     int                    port;
@@ -599,6 +600,7 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
     gboolean               use_ephemeral_port;
     gboolean               use_ikev2;
     gboolean               tls_need_password;
+    gboolean               is_local_set          = FALSE;
     g_autofree char *      pwd_base64            = NULL;
     const char *           tls_key_filename      = NULL;
     const char *           tls_cert_filename     = NULL;
@@ -664,7 +666,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
     value = nm_setting_vpn_get_data_item(s_vpn, NM_L2TP_KEY_USER_AUTH_TYPE);
     if (nm_streq0(value, NM_L2TP_AUTHTYPE_TLS)) {
         priv->user_authtype = TLS_AUTH;
-        crypto_init_openssl();
     }
 
     /**
@@ -679,7 +680,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
             tls_key_filename  = nm_setting_vpn_get_data_item(s_vpn, NM_L2TP_KEY_MACHINE_KEY);
             tls_cert_filename = nm_setting_vpn_get_data_item(s_vpn, NM_L2TP_KEY_MACHINE_CERT);
             tls_ca_filename   = nm_setting_vpn_get_data_item(s_vpn, NM_L2TP_KEY_MACHINE_CA);
-            crypto_init_openssl();
         }
 
         if (priv->ipsec_daemon == NM_L2TP_IPSEC_DAEMON_STRONGSWAN
@@ -701,7 +701,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
                     if (!has_include_ipsec_secrets(ipsec_secrets_file)) {
                         fd = open(ipsec_secrets_file, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);
                         if (fd == -1) {
-                            crypto_deinit_openssl();
                             errsv = errno;
                             snprintf(errorbuf,
                                      sizeof(errorbuf),
@@ -712,7 +711,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
                         }
                         fp = fdopen(fd, "a");
                         if (fp == NULL) {
-                            crypto_deinit_openssl();
                             snprintf(errorbuf,
                                      sizeof(errorbuf),
                                      _("Could not append \"include ipsec.d/ipsec.nm-l2tp.secrets\" "
@@ -735,7 +733,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
                          sizeof(errorbuf),
                          _("Could not write %s/ipsec.nm-l2tp.secrets"),
                          ipsec_conf_dir);
-                crypto_deinit_openssl();
                 return nm_l2tp_ipsec_error(error, errorbuf);
             }
 
@@ -770,14 +767,12 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
             } else { /* TLS_AUTH */
                 if (!tls_key_filename) {
                     close(fd);
-                    crypto_deinit_openssl();
                     return nm_l2tp_ipsec_error(error, _("Machine private key file not supplied"));
                 }
                 tls_key_fileformat =
                     crypto_file_format(tls_key_filename, &tls_need_password, &config_error);
                 if (config_error) {
                     close(fd);
-                    crypto_deinit_openssl();
                     g_propagate_error(error, config_error);
                     return FALSE;
                 }
@@ -792,23 +787,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
                     write_config_option(fd, ": PKCS8");
                     break;
 
-                case NM_L2TP_CRYPTO_FILE_FORMAT_RSA_PKEY_DER:
-                case NM_L2TP_CRYPTO_FILE_FORMAT_RSA_PKEY_PEM:
-                    write_config_option(fd, ": RSA");
-                    break;
-
-                case NM_L2TP_CRYPTO_FILE_FORMAT_DSA_PKEY_DER:
-                case NM_L2TP_CRYPTO_FILE_FORMAT_DSA_PKEY_PEM:
-                    /* strongSwan no longer supports DSA,
-                       we let strongSwan produce an error message */
-                    write_config_option(fd, ": DSA");
-                    break;
-
-                case NM_L2TP_CRYPTO_FILE_FORMAT_ECDSA_PKEY_DER:
-                case NM_L2TP_CRYPTO_FILE_FORMAT_ECDSA_PKEY_PEM:
-                    write_config_option(fd, ": ECDSA");
-                    break;
-
                 default:
                     write_config_option(fd, ": RSA");
                 }
@@ -835,7 +813,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
             crypto_init_nss(NM_IPSEC_NSS_DIR, &config_error);
             if (config_error) {
                 close(fd);
-                crypto_deinit_openssl();
                 g_propagate_error(error, config_error);
                 return FALSE;
             }
@@ -843,7 +820,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
             if (config_error) {
                 close(fd);
                 crypto_deinit_nss(NULL);
-                crypto_deinit_openssl();
                 g_propagate_error(error, config_error);
                 return FALSE;
             }
@@ -865,7 +841,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
             }
             if (config_error) {
                 crypto_deinit_nss(NULL);
-                crypto_deinit_openssl();
                 g_propagate_error(error, config_error);
                 return FALSE;
             }
@@ -873,7 +848,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
             g_byte_array_free(p12_array, TRUE);
             if (config_error) {
                 crypto_deinit_nss(NULL);
-                crypto_deinit_openssl();
                 g_propagate_error(error, config_error);
                 return FALSE;
             }
@@ -887,7 +861,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
         fd       = open(filename, O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
         g_free(filename);
         if (fd == -1) {
-            crypto_deinit_openssl();
             return nm_l2tp_ipsec_error(error, _("Could not write ipsec config"));
         }
 
@@ -915,7 +888,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
                 tls_ca_fileformat = crypto_file_format(tls_ca_filename, NULL, &config_error);
                 if (config_error) {
                     close(fd);
-                    crypto_deinit_openssl();
                     g_propagate_error(error, config_error);
                     return FALSE;
                 }
@@ -946,9 +918,7 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
         }
 
         write_config_option(fd, "  left=%%defaultroute\n");
-        if (use_ephemeral_port) {
-            write_config_option(fd, "  leftprotoport=udp/%%any\n");
-        } else {
+        if (!use_ephemeral_port) {
             write_config_option(fd, "  leftprotoport=udp/l2tp\n");
         }
         if (priv->ipsec_daemon == NM_L2TP_IPSEC_DAEMON_LIBRESWAN
@@ -966,7 +936,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
                     &config_error);
                 if (config_error) {
                     close(fd);
-                    crypto_deinit_openssl();
                     g_propagate_error(error, config_error);
                     return FALSE;
                 }
@@ -980,7 +949,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
             } else {
                 if (!tls_key_filename) {
                     close(fd);
-                    crypto_deinit_openssl();
                     return nm_l2tp_ipsec_error(error, _("Machine certificate file not supplied"));
                 }
                 write_config_option(fd, "  leftcert=\"%s\"\n", tls_cert_filename);
@@ -1094,7 +1062,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
         g_free(filename);
 
         if (fd == -1) {
-            crypto_deinit_openssl();
             return nm_l2tp_ipsec_error(error, _("Could not write kl2tpd config."));
         }
 
@@ -1114,7 +1081,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
         g_free(filename);
 
         if (fd == -1) {
-            crypto_deinit_openssl();
             return nm_l2tp_ipsec_error(error, _("Could not write xl2tpd config."));
         }
 
@@ -1122,6 +1088,10 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
         write_config_option(fd, "access control = yes\n");
 
         write_config_option(fd, "port = %d\n", port);
+        if (getenv("NM_L2TP_XL2TPD_MAX_RETRIES")) {
+            max_retries = strtol(getenv("NM_L2TP_XL2TPD_MAX_RETRIES"), NULL, 10);
+            write_config_option(fd, "max retries = %ld\n", max_retries);
+        }
         if (_LOGD_enabled()) {
             /* write_config_option (fd, "debug network = yes\n"); */
             write_config_option(fd, "debug state = yes\n");
@@ -1152,7 +1122,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
     g_free(filename);
 
     if (fd == -1) {
-        crypto_deinit_openssl();
         return nm_l2tp_ipsec_error(error, _("Could not write ppp options."));
     }
 
@@ -1161,19 +1130,59 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
 
     write_config_option(fd, "ipparam nm-l2tp-service-%s\n", priv->uuid);
 
-    /* pass gateway IP address to nm-l2tp-pppd-plugin via ipcp_wantoptions[0].hisaddr,
-       but let pppd use the remote IP address being offered by the peer using IPCP */
-    write_config_option(fd, ":%s\n", priv->saddr);
-    write_config_option(fd, "ipcp-accept-remote\n");
-
     write_config_option(fd, "nodetach\n");
 
+    /* Any IPv4 configuration options */
     s_ip4 = nm_connection_get_setting_ip4_config(priv->connection);
-    if (!nm_setting_ip_config_get_ignore_auto_dns(s_ip4)) {
-        write_config_option(fd, "usepeerdns\n");
+    if (s_ip4) {
+
+        value = nm_setting_ip_config_get_method (s_ip4);
+        if (nm_streq0(value, NM_SETTING_IP4_CONFIG_METHOD_MANUAL)) {
+            const char *ipv4_str = NULL;
+            const char *gway_str = NULL;
+            const char *mask_str = NULL;
+            char buf[NM_UTILS_INET_ADDRSTRLEN];
+            NMIPAddress *ipv4 = NULL;
+
+            /* If <local:remote> is specified, the IPCP negotiation will fail unless
+             *   - ipcp-accept-local, and/or
+             *   - ipcp-accept-remote
+             * is specified. That depends on the server, but in any case allow it.
+             *
+             * The "manual" option is really just a suggestion. "auto" is the default.
+             */
+            ipv4 = nm_setting_ip_config_get_address(s_ip4, 0);
+            if (ipv4) {
+                int prefix = nm_ip_address_get_prefix(ipv4);
+                ipv4_str = nm_ip_address_get_address(ipv4);
+                mask_str = nm_utils_inet4_ntop(nm_utils_ip4_prefix_to_netmask(prefix), buf);
+
+                gway_str = nm_setting_ip_config_get_gateway(s_ip4);
+                if (ipv4_str && gway_str) {
+                    write_config_option(fd, "%s:%s\n", ipv4_str, gway_str);
+                    if (mask_str) {
+                        write_config_option(fd, "netmask %s\n", mask_str);
+                    }
+                    write_config_option(fd, "ipcp-accept-local\n");
+                    write_config_option(fd, "ipcp-accept-remote\n");
+                    is_local_set = TRUE;
+                }
+            }
+        }
+        if (nm_streq (value, NM_SETTING_IP4_CONFIG_METHOD_DISABLED)) {
+            write_config_option(fd, "noip\n");
+        } else {
+            if (!nm_setting_ip_config_get_ignore_auto_dns(s_ip4)) {
+                write_config_option(fd, "usepeerdns\n");
+            }
+        }
     }
 
-    write_config_option(fd, "noipdefault\n");
+    if (!is_local_set) {
+        write_config_option(fd, "noipdefault\n");
+    }
+    is_local_set = FALSE;
+
     write_config_option(fd, "nodefaultroute\n");
 
     /* Don't need to auth the L2TP server */
@@ -1199,7 +1208,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
         tls_key_fileformat = crypto_file_format(tls_key_filename, &tls_need_password, error);
         if (*error) {
             close(fd);
-            crypto_deinit_openssl();
             return FALSE;
         }
         if (tls_need_password)
@@ -1222,19 +1230,14 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
                                        error);
             if (*error) {
                 close(fd);
-                crypto_deinit_openssl();
                 return FALSE;
             }
         } else {
             switch (tls_key_fileformat) {
             case NM_L2TP_CRYPTO_FILE_FORMAT_PKCS8_DER:
-            case NM_L2TP_CRYPTO_FILE_FORMAT_RSA_PKEY_DER:
-            case NM_L2TP_CRYPTO_FILE_FORMAT_DSA_PKEY_DER:
-            case NM_L2TP_CRYPTO_FILE_FORMAT_ECDSA_PKEY_DER:
                 crypto_pkey_der_to_pem_file(tls_key_filename, value, tls_key_out_filename, error);
                 if (*error) {
                     close(fd);
-                    crypto_deinit_openssl();
                     return FALSE;
                 }
                 break;
@@ -1247,14 +1250,12 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
             tls_cert_fileformat = crypto_file_format(tls_cert_filename, NULL, error);
             if (*error) {
                 close(fd);
-                crypto_deinit_openssl();
                 return FALSE;
             }
             if (tls_cert_fileformat == NM_L2TP_CRYPTO_FILE_FORMAT_X509_DER) {
                 crypto_x509_der_to_pem_file(tls_cert_filename, tls_cert_out_filename, error);
                 if (*error) {
                     close(fd);
-                    crypto_deinit_openssl();
                     return FALSE;
                 }
             } else {
@@ -1266,14 +1267,12 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
                 tls_ca_fileformat = crypto_file_format(tls_ca_filename, NULL, error);
                 if (*error) {
                     close(fd);
-                    crypto_deinit_openssl();
                     return FALSE;
                 }
                 if (tls_ca_fileformat == NM_L2TP_CRYPTO_FILE_FORMAT_X509_DER) {
                     crypto_x509_der_to_pem_file(tls_ca_filename, tls_ca_out_filename, error);
                     if (*error) {
                         close(fd);
-                        crypto_deinit_openssl();
                         return FALSE;
                     }
                 } else {
@@ -1376,7 +1375,6 @@ nm_l2tp_config_write(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
     }
 
     close(fd);
-    crypto_deinit_openssl();
     return TRUE;
 }
 
@@ -1562,7 +1560,7 @@ nm_l2tp_start_ipsec(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
             if (!WEXITSTATUS(status)) {
                 if (priv->ipsec_daemon == NM_L2TP_IPSEC_DAEMON_LIBRESWAN) {
                     rc = TRUE;
-                    _LOGI("Libreswan IPsec tunnel is up.");
+                    g_message("Libreswan IPsec connection is up.");
                 } else {
                     /* Do not trust exit status of strongSwan 'ipsec up' command.
                        explictly check if connection is established.
@@ -1574,10 +1572,11 @@ nm_l2tp_start_ipsec(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
                              priv->ipsec_binary_path,
                              priv->uuid);
                     if (g_spawn_command_line_sync(cmdbuf, &output, NULL, NULL, NULL)) {
-                        rc = output && strstr(output, "ESTABLISHED");
+                        rc = output && strstr(output, "ESTABLISHED") &&
+                             strstr(output, "INSTALLED, TRANSPORT");
                         g_free(output);
                         if (rc) {
-                            _LOGI("strongSwan IPsec tunnel is up.");
+                            g_message("strongSwan IPsec connection is up.");
                         }
                     }
                 }
@@ -1590,7 +1589,7 @@ nm_l2tp_start_ipsec(NML2tpPlugin *plugin, NMSettingVpn *s_vpn, GError **error)
             snprintf(cmdbuf, sizeof(cmdbuf), "%s stop", priv->ipsec_binary_path);
             sys = system(cmdbuf);
         }
-        _LOGW("Could not establish IPsec tunnel.");
+        g_message("Could not establish IPsec connection.");
     }
 
     return rc;
@@ -1697,9 +1696,6 @@ handle_need_secrets(NMDBusL2tpPpp *object, GDBusMethodInvocation *invocation, gp
         switch (tls_key_fileformat) {
         case NM_L2TP_CRYPTO_FILE_FORMAT_PKCS12:
         case NM_L2TP_CRYPTO_FILE_FORMAT_PKCS8_DER:
-        case NM_L2TP_CRYPTO_FILE_FORMAT_RSA_PKEY_DER:
-        case NM_L2TP_CRYPTO_FILE_FORMAT_DSA_PKEY_DER:
-        case NM_L2TP_CRYPTO_FILE_FORMAT_ECDSA_PKEY_DER:
             key_filename = g_strdup_printf(RUNSTATEDIR "/nm-l2tp-%s/key.pem", priv->uuid);
             break;
 
@@ -1804,7 +1800,10 @@ handle_set_ip4_config(NMDBusL2tpPpp *        object,
     /* Insert the external VPN gateway into the table, which the pppd plugin
      * simply doesn't know about.
      */
-    g_variant_builder_add(&builder, "{sv}", NM_L2TP_KEY_GATEWAY, g_variant_new_uint32(priv->naddr));
+    g_variant_builder_add(&builder,
+                          "{sv}",
+                          NM_VPN_PLUGIN_CONFIG_EXT_GATEWAY,
+                          g_variant_new_uint32(priv->naddr));
     new_config = g_variant_builder_end(&builder);
     g_variant_ref_sink(new_config);
 
@@ -2293,12 +2292,14 @@ main(int argc, char *argv[])
 {
     NML2tpPlugin *   plugin;
     GMainLoop *      main_loop;
-    gboolean         persist       = FALSE;
-    GOptionContext * opt_ctx       = NULL;
-    GError *         error         = NULL;
-    g_autofree char *bus_name_free = NULL;
+    gboolean         persist               = FALSE;
+    GOptionContext * opt_ctx               = NULL;
+    GError *         error                 = NULL;
+    g_autofree char *bus_name_free         = NULL;
     const char *     bus_name;
     char             sbuf[30];
+    char *           l2tp_ppp_module[]     = { "/sbin/modprobe", "l2tp_ppp", NULL };
+    char *           l2tp_netlink_module[] = { "/sbin/modprobe", "l2tp_netlink", NULL };
 
     GOptionEntry options[] = {{"persist",
                                0,
@@ -2376,6 +2377,24 @@ main(int argc, char *argv[])
     if (!persist)
         g_signal_connect(plugin, "quit", G_CALLBACK(quit_mainloop), main_loop);
 
+    if (getenv("NM_L2TP_MODPROBE")) {
+        /* Fedora and RHEL have moved the L2TP kernel modules to the
+         * 'kernel-modules-extra' package and blacklisted all modules from
+         * the 'kernel-modules-extra' package by default.
+         * Load the L2TP modules now. Ignore errors.
+         * https://access.redhat.com/articles/3760101
+         */
+        if (!g_spawn_sync(NULL, l2tp_ppp_module, NULL, 0, NULL, NULL, NULL, NULL, NULL, &error)) {
+            _LOGW("modprobing l2tp_ppp failed: %s", error->message);
+            g_error_free(error);
+        }
+
+        if (!g_spawn_sync(NULL, l2tp_netlink_module, NULL, 0, NULL, NULL, NULL, NULL, NULL, &error)) {
+            _LOGW("modprobing l2tp_netlink failed: %s", error->message);
+            g_error_free(error);
+        }
+    }
+
     g_main_loop_run(main_loop);
 
     g_main_loop_unref(main_loop);
